﻿/*	ATtiny2313
	aitendo/ZY-FGD1442701V1(ST7735) 制御テスト
	『昼夜逆転』工作室	http://jsdiy.web.fc2.com

参照設定から .NETタブ - System.Drawing を追加すること。

【更新履歴】
2013/05/27	ver 1.00	初版（キャラクタデータ転送版）
2013/06/04	ver 2.00	バイナリデータ転送版に改造
*/

using System;
using System.IO;	//File
using System.Diagnostics;	//Stopwatch
using ModifiedSerialPort;

namespace lcdST7735com
{
	//エラーコード
	enum ErrorCode
	{
		NormalTermination = 0,
		COM_Error,
		Error = 255
	};

	//オプションで指定される値
	struct App
	{
		public static string portName = null, imgFile = null;
	}

	class Program
	{
		private const string APP_VER = "v2.00";
		private static MSerialPort comCtrl = null;
		private static ST7735Ctrl lcdCtrl = null;
		private static string[] COMPortNameList = null;
		private static Stopwatch sw = new Stopwatch();

		static void Main(string[] args)
		{
			//UnhandledExceptionイベント・ハンドラを登録する
			AppDomain.CurrentDomain.UnhandledException += new UnhandledExceptionEventHandler(Application_UnhandledException);

			//コマンドラインオプションを取得する
			string arg;
			for (int i = 0; i < args.Length; i++)
			{
				arg = args[i].ToLower();
				if (arg.StartsWith("-com"))
					App.portName = "COM" + args[i].Substring(4);
				else if (arg.EndsWith(".jpg"))
					App.imgFile = arg;
			}
			
			//COMポート番号と画像ファイルが指定されなかった場合は終了する
			if (string.IsNullOrEmpty(App.portName) || string.IsNullOrEmpty(App.imgFile))
			{
				COMPortNameList = MSerialPort.GetPortNames();
				ShowHelp();
				AppClose(null, ErrorCode.NormalTermination);
			}

			if (!File.Exists(App.imgFile))
				AppClose("ファイルが見つかりません。: " + App.imgFile, ErrorCode.Error);

			//シリアルポートをオープンする
			//デフォルト設定：COM1, 9600bps, Parity.NONE, Data=8bit, StopBits.One
			//ReadLine()で解釈される改行コード：SerialPort.NewLine="\n"
			comCtrl = new MSerialPort();
			comCtrl.ReadTimeout = 2000;		//(ms)
			comCtrl.WriteTimeout = 2000;	//(ms)
			comCtrl.BaudRate = 500000;	//AVR 8MHzでのUSART最高速
			comCtrl.Parity = System.IO.Ports.Parity.Even;
			comCtrl.StopBits = System.IO.Ports.StopBits.Two;	//高速通信ではストップビット2bitで安定化を図る
			comCtrl.PortName = App.portName;
			try { comCtrl.Open(); }
			catch { AppClose(comCtrl.PortName + " open error.", ErrorCode.COM_Error); }

			//デバイスの接続を確認する
			lcdCtrl = new ST7735Ctrl();
			if (lcdCtrl.DeviceContact(comCtrl))
				lcdCtrl.SetSerialPort(comCtrl);
			else
				AppClose(comCtrl.PortName + " NG.", ErrorCode.COM_Error);
			GC.Collect();

			//LCDを初期化する
			lcdCtrl.Init();
			lcdCtrl.Clear(0xFFFF);	//パッと見の動作確認のため、画面を白／黒で塗りつぶしてみる
			lcdCtrl.Clear(0x0000);

			sw.Reset();
			sw.Start();
			
			lcdCtrl.DrawImage(App.imgFile);
			
			sw.Stop();
			Console.WriteLine("DrawImage({0}) {1:f1}sec", Path.GetFileName(App.imgFile), sw.Elapsed.TotalSeconds);

			//アプリ正常終了（COMポートのクローズ処理込み）
			AppClose(null, ErrorCode.NormalTermination);
		}
		//end of Main()

		//未処理の例外をキャッチする
		static void Application_UnhandledException(object sender, UnhandledExceptionEventArgs e)
		{
			Console.WriteLine("予期しないエラーが発生しました。アプリを終了します。");
		}

		//アプリ終了メッセージ
		private static void AppClose(string mes, ErrorCode code)
		{
			//終了メッセージを表示する
			if (mes != null) Console.WriteLine(mes + Environment.NewLine);

			//COMポート クローズ
			try { comCtrl.Close(); }
			catch { }

			Console.WriteLine("Press any key.");
			Console.ReadKey();

			Environment.Exit((int)code);
		}

		//ヘルプ画面
		private static void ShowHelp()
		{
			Console.WriteLine("ZY-FGD1442701V1-LCD(ST7735) Control Test {0:s}", APP_VER);
			Console.WriteLine("(C)jsdiy/2013  http://jsdiy.web.fc2.com");
			Console.WriteLine();
			Console.WriteLine(
				"[Usage]" + Environment.NewLine +
				">lcdST7735com.exe imagefile.jpg -com<n>" + Environment.NewLine +
				Environment.NewLine +
				"[Options]" + Environment.NewLine +
				"-com<n> : COMポート番号  例: -com2"
				);
			Console.WriteLine();
			Console.WriteLine("[Serial port]");
			for (int i = 0; i < COMPortNameList.Length; i++)
				Console.Write(COMPortNameList[i] + " ");
			Console.WriteLine();
			Console.WriteLine();
		}

	}	//class
}	//namespace
